/*
 * kicker_data.h
 *
 *  Created on: 16.03.2011
 *      Author: AndreR
 */

#ifndef __KICKER_DATA_H__
#define __KICKER_DATA_H__

#include <stdint.h>
#include "fixed.h"

#define KICKER_DATA_SIZE 16

#define KICKER_CMD_NONE			0x00
#define KICKER_CMD_OFF			0x01
#define KICKER_CMD_LOAD_MANUAL	0x02
#define KICKER_CMD_LOAD_AUTO	0x04
#define KICKER_CMD_FIRE			0x08
#define KICKER_CMD_SET_FAN		0x10

#define KICKER_DEVICE_STRAIGHT	0
#define KICKER_DEVICE_CHIP		1

#define KICKER_FAN_MANUAL		0
#define KICKER_FAN_AUTO			1

typedef union __attribute__ ((packed)) _KickerData
{
	struct
	{
		uint16_t cmd;

		union
		{
			struct
			{
				uint16_t tOnTicks;
				uint16_t tOffTicks;
				uint16_t duration; // in 100us steps
			} loadManual;

			struct
			{
				uint16_t maxCapLevel;
				uint8_t reserved[4];
			} loadAuto;

			struct
			{
				FixedPoint142 duration;	// [s*e-7]
				uint8_t tOn;	// 0 - 255
				uint8_t device;
				uint8_t reserved[2];
			} fire;

			struct
			{
				uint8_t mode;
				uint8_t speed;
				uint8_t reserved[4];
			} setFan;
		} cmds;

		uint16_t uCap;
		uint16_t iChg;
		uint16_t TIGBT[2];
	};

	struct
	{
		uint8_t mosi[KICKER_DATA_SIZE/2];
		uint8_t miso[KICKER_DATA_SIZE/2];
	} exchange;
} KickerData;

#endif
