/*
 * buttons.c
 *
 *  Created on: 29.05.2010
 *      Author: AndreR
 */

#include "exti.h"

#include "stm32f10x.h"
#include "misc.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_exti.h"
#include "stm32f10x_gpio.h"

#include "constants.h"

#define BUTTON_PIN1	(1 << 8)	// Port A
#define BUTTON_PIN2	(1 << 13)	// Port C

Exti exti;

void EXTI9_5_IRQHandler()
{
	if(EXTI->PR & BUTTON_PIN1)
	{
		if(GPIOA->IDR & BUTTON_PIN1)
		{
			// high level => released
			exti.buttons[0].pressed = 0;
			exti.buttons[0].released = 1;
		}
		else
		{
			exti.buttons[0].pressed = 1;
			exti.buttons[0].released = 0;
		}

		EXTI->PR = BUTTON_PIN1;
	}
}

void EXTI15_10_IRQHandler()
{
	if(EXTI->PR & BUTTON_PIN2)
	{
		if(GPIOC->IDR & BUTTON_PIN2)
		{
			// high level => released
			exti.buttons[1].pressed = 0;
			exti.buttons[1].released = 1;
		}
		else
		{
			exti.buttons[1].pressed = 1;
			exti.buttons[1].released = 0;
		}

		EXTI->PR = BUTTON_PIN2;
	}

	if(EXTI->PR & IR_IRQ)
	{
		if(exti.callback)
			(*exti.callback)();

		EXTI->PR = IR_IRQ;
	}
}

void ExtiInit()
{
	exti.buttons[0].pressed = 0;
	exti.buttons[0].released = 0;
	exti.buttons[1].pressed = 0;
	exti.buttons[1].released = 0;
	exti.callback = 0;

	// Button 1
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Pin = GPIO_Pin_8;
	gpioInit.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOA, &gpioInit);

	GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource8);

	EXTI_InitTypeDef EXTIInit;
	EXTI_StructInit(&EXTIInit);
	EXTIInit.EXTI_Line = EXTI_Line8;
	EXTIInit.EXTI_LineCmd = ENABLE;
	EXTIInit.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTIInit.EXTI_Trigger = EXTI_Trigger_Falling;
	EXTI_Init(&EXTIInit);

	NVIC_InitTypeDef NVICInit;
	NVICInit.NVIC_IRQChannel = EXTI9_5_IRQn;
	NVICInit.NVIC_IRQChannelCmd = ENABLE;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_BUTTON1;	//lowest priority
	NVICInit.NVIC_IRQChannelSubPriority = 0;	//lowest priority
	NVIC_Init(&NVICInit);

	// Button 2
	gpioInit.GPIO_Pin = GPIO_Pin_13;
	gpioInit.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOC, &gpioInit);

	GPIO_EXTILineConfig(GPIO_PortSourceGPIOC, GPIO_PinSource13);

	EXTI_StructInit(&EXTIInit);
	EXTIInit.EXTI_Line = EXTI_Line13;
	EXTIInit.EXTI_LineCmd = ENABLE;
	EXTIInit.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTIInit.EXTI_Trigger = EXTI_Trigger_Falling;
	EXTI_Init(&EXTIInit);

	// IR IRQ
	gpioInit.GPIO_Pin = IR_IRQ;
	gpioInit.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(GPIOF, &gpioInit);

	GPIO_EXTILineConfig(GPIO_PortSourceGPIOF, GPIO_PinSource15);

	EXTI_StructInit(&EXTIInit);
	EXTIInit.EXTI_Line = EXTI_Line15;
	EXTIInit.EXTI_LineCmd = ENABLE;
	EXTIInit.EXTI_Mode = EXTI_Mode_Interrupt;
	EXTIInit.EXTI_Trigger = EXTI_Trigger_Falling;
	EXTI_Init(&EXTIInit);

	NVICInit.NVIC_IRQChannel = EXTI15_10_IRQn;	// interrupts must be configured separately
	NVICInit.NVIC_IRQChannelCmd = ENABLE;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_BUTTON2_AND_IR;	//lowest priority
	NVICInit.NVIC_IRQChannelSubPriority = 0;	//lowest priority
	NVIC_Init(&NVICInit);
}

void ExtiSetIRCallback(IRCallback cb)
{
	exti.callback = cb;
}

uint8_t ButtonsIsPressed(uint8_t id)
{
	if(id >= 2)
		return 0;

	if(exti.buttons[id].pressed)
	{
		exti.buttons[id].pressed = 0;
		return 1;
	}

	return 0;
}

uint8_t ButtonsIsReleased(uint8_t id)
{
	if(id >= 2)
		return 0;

	if(exti.buttons[id].released)
	{
		exti.buttons[id].released = 0;
		return 1;
	}

	return 0;
}
