/*
 * lis3.c
 *
 *  Created on: 07.03.2011
 *      Author: AndreR
 */

#include "lis3.h"

#define LIS3_READ 	(1 << 7)
#define LIS3_WRITE 	(0 << 7)
#define LIS3_SINGLE (0 << 6)
#define LIS3_MULTI	(1 << 6)

#define CMD_SET_CONFIGURATION 1
#define CMD_GET_ACCELERATION 2

void SPILIS3Callback(SPIHeader* pHeader, uint8_t* pData, void* pUserData)
{
	LIS3* pLIS3 = (LIS3*)pUserData;

	if(pHeader->slaveId != pLIS3->slaveId)
		return;

	if(pHeader->cmd == CMD_GET_ACCELERATION)
	{
		for(uint8_t i = 0; i < 3; i++)
			pLIS3->raw[i] = *((int16_t*)(pData + i*2 + 1));
	}
}

int16_t SPILIS3Init(LIS3* pLIS3, SPI* pSPI, GPIO_TypeDef* pPort, uint16_t pin)
{
	pLIS3->raw[0] = 0;
	pLIS3->raw[1] = 0;
	pLIS3->raw[2] = 0;

	pLIS3->pSPI = pSPI;

	SPISlave slave;
	slave.callback = &SPILIS3Callback;
	slave.pUserData = pLIS3;
	slave.pCSPort = pPort;
	slave.csPin = pin;

	slave.prescaler = SPI_BaudRatePrescaler_256;
	slave.cpol = 0;	// idle high
	slave.cpha = 0;	// sample on second clock transition

	return SPIAddSlave(pSPI, slave, &pLIS3->slaveId);
}

int16_t SPILIS3SetConfiguration(LIS3* pLIS3, uint8_t enable)
{
	SPIHeader header;
	header.slaveId = pLIS3->slaveId;
	header.cmd = CMD_SET_CONFIGURATION;
	header.length = 2;

	uint8_t data[2];
	data[0] = 0x20 | LIS3_WRITE | LIS3_SINGLE;	//CTRL_REG1
	data[1] = 0;
	if(enable)
		data[1] |= (0b11000111);

	return SPIPutPacket(pLIS3->pSPI, &header, 2, data);
}

int16_t SPILIS3GetAcceleration(LIS3* pLIS3)
{
	SPIHeader header;
	header.slaveId = pLIS3->slaveId;
	header.cmd = CMD_GET_ACCELERATION;
	header.length = 7;

	uint8_t data = 0x28 | LIS3_READ | LIS3_MULTI;

	return SPIPutPacket(pLIS3->pSPI, &header, 1, &data);
}
