/*
 * ir.c
 *
 *  Created on: 04.07.2010
 *      Author: AndreR
 */

#include "ir.h"

#include "stm32f10x.h"
#include "misc.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_gpio.h"

#include "FreeRTOS.h"
#include "task.h"

#include "spi1_2.h"
#include "kicker.h"
#include "hal/exti.h"
#include "network.h"
#include "util/mem.h"

#include "atmega/ir_prog.h"

#define IR_ATMEGA_CS	GPIO_Pin_10	// GPIOE
#define IR_ATMEGA_RST	GPIO_Pin_3	// GPIOA

IRGlobal ir;

// called from ISR!
void IRIRQ()
{
	// IR barrier interrupted
	signed portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

	xSemaphoreGiveFromISR(ir.hRun, &xHigherPriorityTaskWoken);

	if(xHigherPriorityTaskWoken == pdTRUE)
		taskYIELD();
}

void IRInit()
{
	GPIO_InitTypeDef gpioInit;

	gpioInit.GPIO_Mode = GPIO_Mode_Out_PP;
	gpioInit.GPIO_Pin = IR_ATMEGA_RST;
	gpioInit.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOA, &gpioInit);

	gpioInit.GPIO_Pin = IR_ATMEGA_CS;
	GPIO_Init(GPIOE, &gpioInit);

	GPIOA->BSRR = IR_ATMEGA_RST;
	GPIOE->BSRR = IR_ATMEGA_CS;

	ATMegaInit(&ir.atmega, &spi1, GPIOE, IR_ATMEGA_CS, GPIOA, IR_ATMEGA_RST);

	ExtiSetIRCallback(&IRIRQ);

	vSemaphoreCreateBinary(ir.hRun);

	ir.logEnabled = 0;
}

void IRTask(void* params)
{
	params = params;

	portTickType xLastWakeTime = xTaskGetTickCount();
	portTickType xLastStatusTime = xLastWakeTime;

	while(1)
	{
		xLastWakeTime = xTaskGetTickCount();

		xSemaphoreTake(ir.hRun, 100/portTICK_RATE_MS);

		ATMegaExchange(&ir.atmega, ir.data.exchange.mosi, ir.data.exchange.miso, IR_DATA_SIZE);

		if(!(GPIOF->IDR & IR_IRQ))	// barrier interrupted?
		{
			if(ir.state == IR_STATE_ARMED)
			{
				KickerFire(ir.fireDuration, ir.level, ir.device);

				ir.state = IR_STATE_STANDBY;
			}
		}

		if(xLastWakeTime - xLastStatusTime > 50/portTICK_RATE_MS && ir.logEnabled)
		{
			xLastStatusTime = xLastWakeTime;

			KickerIrLog log;
			for(uint8_t i = 0; i < 4; i++)
				log.voltage[i] = ir.data.level[i];

			PacketHeader header;
			header.section = SECTION_KICKER;
			header.cmd = CMD_KICKER_IR_LOG;
			header.len = LEN_KICKER_IR_LOG;

			NetworkSendPacket(&header, (uint8_t*)&log, 0);
		}
	}
}

void IRSetLog(uint8_t enable)
{
	ir.logEnabled = enable;
}

void IRArm(uint8_t device, uint16_t fireDuration, uint8_t level)
{
	ir.fireDuration = fireDuration;
	ir.level = level;
	ir.device = device;
	ir.state = IR_STATE_ARMED;
}

void IRDisarm()
{
	ir.fireDuration = 0;
	ir.state = IR_STATE_STANDBY;
}
