/*
 * kicker.c
 *
 *  Created on: 19.03.2011
 *      Author: AndreR
 */

#include "kicker.h"

#include "FreeRTOS.h"
#include "task.h"

#include "network.h"
#include "commands.h"
#include "spi1_2.h"
#include "util/critical.h"
#include "ir.h"

#include "atmega/kicker_prog.h"

#define KICKER_ATMEGA_RST	GPIO_Pin_5	// GPIOB
#define KICKER_ATMEGA_CS	GPIO_Pin_8	// GPIOB

Kicker kicker;

void KickerInit()
{
	GPIO_InitTypeDef gpioInit;

	gpioInit.GPIO_Mode = GPIO_Mode_Out_PP;
	gpioInit.GPIO_Pin = KICKER_ATMEGA_RST | KICKER_ATMEGA_CS;
	gpioInit.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_Init(GPIOB, &gpioInit);

	GPIOB->BSRR = KICKER_ATMEGA_RST | KICKER_ATMEGA_CS;

	ATMegaInit(&kicker.atmega, &spi1, GPIOB, KICKER_ATMEGA_CS, GPIOB, KICKER_ATMEGA_RST);

	kicker.data.cmd = 0;

	kicker.logEnabled = 1;

	kicker.latestStatus.TDiode[0] = 0;
	kicker.latestStatus.TDiode[1] = 0;
	kicker.latestStatus.TIGBT[0] = 0;
	kicker.latestStatus.TIGBT[1] = 0;
	kicker.latestStatus.cap = 0;
	kicker.latestStatus.chg = 0;

	kicker.hDataInUse = xSemaphoreCreateMutex();
	vSemaphoreCreateBinary(kicker.hNewCommand);
}

void KickerSetLogging(uint8_t enable)
{
	kicker.logEnabled = enable;
}

void KickerLoadManual(uint16_t tOn, uint16_t tOff, uint16_t duration)
{
	TASK_MUTEX(kicker.hDataInUse)
	{
		kicker.data.cmds.loadManual.tOnTicks = tOn;
		kicker.data.cmds.loadManual.tOffTicks = tOff;
		kicker.data.cmds.loadManual.duration = duration;
		kicker.data.cmd = KICKER_CMD_LOAD_MANUAL;
	}
}

void KickerLoadAuto(uint16_t maxCap)
{
	TASK_MUTEX(kicker.hDataInUse)
	{
		kicker.data.cmds.loadAuto.maxCapLevel = maxCap;
		kicker.data.cmd = KICKER_CMD_LOAD_AUTO;
	}
}

void KickerFire(uint16_t duration, uint8_t tOn, uint8_t device)
{
	IRDisarm();

	TASK_MUTEX(kicker.hDataInUse)
	{
		kicker.data.cmds.fire.duration._data = duration;
		kicker.data.cmds.fire.tOn = tOn;
		kicker.data.cmds.fire.device = device;
		kicker.data.cmd = KICKER_CMD_FIRE;
	}

	xSemaphoreGive(kicker.hNewCommand);
}

void KickerSetFan(uint8_t mode, uint8_t speed)
{
	TASK_MUTEX(kicker.hDataInUse)
	{
		kicker.data.cmds.setFan.mode = mode;
		kicker.data.cmds.setFan.speed = speed;
		kicker.data.cmd = KICKER_CMD_SET_FAN;
	}
}

void KickerTask(void* params)
{
	params = params;

	portTickType xLastWakeTime = xTaskGetTickCount();
	portTickType xLastStatusTime = xLastWakeTime;

	while(1)
	{
		xLastWakeTime = xTaskGetTickCount();

		xSemaphoreTake(kicker.hNewCommand, 50/portTICK_RATE_MS);

		TASK_MUTEX(kicker.hDataInUse)
		{
			ATMegaExchange(&kicker.atmega, kicker.data.exchange.mosi, kicker.data.exchange.miso, KICKER_DATA_SIZE);

			kicker.data.cmd = KICKER_CMD_NONE;
		}

		if(xLastWakeTime - xLastStatusTime > 50/portTICK_RATE_MS && kicker.logEnabled)
		{
			xLastStatusTime = xLastWakeTime;

			kicker.latestStatus.chg = kicker.data.iChg;
			kicker.latestStatus.TDiode[0] = 0;
			kicker.latestStatus.TDiode[1] = 0;
			kicker.latestStatus.TIGBT[0] = kicker.data.TIGBT[0];
			kicker.latestStatus.TIGBT[1] = kicker.data.TIGBT[1];
			kicker.latestStatus.cap = kicker.data.uCap;

			PacketHeader header;
			header.section = SECTION_KICKER;
			header.cmd = CMD_KICKER_STATUSV2;
			header.len = LEN_KICKER_STATUSV2;

			NetworkSendPacket(&header, (uint8_t*)&kicker.latestStatus, 0);
		}
	}
}

