/*
 * dect_transceiver.h
 *
 *  Created on: 13.02.2011
 *      Author: AndreR
 */

#ifndef __DECTINTERFACE_H__
#define __DECTINTERFACE_H__

#include "constants.h"
#include "util/fifo_packet_synced.h"
#include "util/fifo8.h"
#include "errors.h"

#include "FreeRTOS.h"
#include "semphr.h"

#define DECT_MODE_CONFIG	0
#define DECT_MODE_DATA		1
#define DECT_MODE_TRANSITION 2

typedef void(*DectRxCallback)();

typedef struct _DECTStatistic
{
	struct
	{
		volatile uint32_t packets;
		volatile uint32_t drops;
		volatile uint32_t bytes;
	} rx;

	struct
	{
		volatile uint32_t packets;
		volatile uint32_t drops;
		volatile uint32_t bytes;
	} tx;
} DECTStatistic;

typedef struct _DECTInterface
{
	uint8_t mode;

	DectRxCallback rxCallback;

	Fifo8 rxConfig;	// receive buffer in config mode
	uint8_t rxState;

	uint8_t frameNum;

	DECTStatistic stats;

	struct
	{
		uint8_t dmaTxBuf[DECT_DMA_TX_SIZE];

		FifoPacketSynced tx;	// Storing HDLC frames
		FifoPacketSynced rx;	// here too

		uint8_t txData[DECT_HDLC_TX_SIZE];
		uint8_t rxData[DECT_HDLC_RX_SIZE];
	} HDLC;

	xSemaphoreHandle txDataInUse;
	xSemaphoreHandle rxDataInUse;
} DECTInterface;

extern DECTInterface dectInterface;

void	DECTInterfaceInit();
uint8_t DECTInterfaceGetMode();
void	DECTInterfaceSetMode(uint8_t mode);
void	DECTInterfaceSetRxCallback(DectRxCallback cb);
int16_t	DECTInterfaceConfigPut(uint8_t* pSrc, uint16_t len);
int16_t	DECTInterfaceConfigGet(uint8_t* pDst, uint16_t len, uint16_t* pBytesRead);
int16_t DECTInterfaceGet(uint8_t* pData, uint16_t srcLength, uint8_t** ppDecoded, uint16_t* pDecodedLength);
int16_t DECTInterfacePut(uint8_t* pSrc, uint16_t); // put HDLC frame
void	DECTResetStats();

#endif /* DECT_TRANSCEIVER_H_ */
