/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - RCU
 * Date: 20.10.2010
 * Author(s): Lukas
 * 
 * *********************************************************
 */

package edu.dhbw.mannheim.tigers.robotcontrolutility.model;

import java.io.*;
import java.util.*;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.robotcontrolutility.presenter.AControllerPresenter;
import edu.dhbw.mannheim.tigers.robotcontrolutility.presenter.RCUPresenter;
import edu.dhbw.mannheim.tigers.robotcontrolutility.presenter.AControllerPresenter.EControllerType;


/**
 * This class manages saving and loading of configs.
 * 
 * @author Lukas
 * 
 */

public class ConfigManager
{
	
	// --------------------------------------------------------------------------
	// --- class variables ------------------------------------------------------
	// --------------------------------------------------------------------------
	private static ConfigManager						instance							= null;
	private static final HashMap<String, String>	defaultGamePadConfigMap		= new HashMap<String, String>();
	private static final HashMap<String, String>	defaultKeyboardConfigMap	= new HashMap<String, String>();
	
	// --- logger ---
	private Logger											log								= Logger.getLogger(RCUPresenter.class
																											.getName());
	
	
	// --------------------------------------------------------------------------
	// --- constructor(s) -------------------------------------------------------
	// --------------------------------------------------------------------------
	private ConfigManager()
	{
		this.setDefaultConfig();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	public static ConfigManager getInstance()
	{
		if (instance == null)
			instance = new ConfigManager();
		return instance;
	}
	

	private void setDefaultConfig()
	{
		defaultGamePadConfigMap.put("-y", "forward");
		defaultGamePadConfigMap.put("y", "backward");
		defaultGamePadConfigMap.put("-x", "left");
		defaultGamePadConfigMap.put("x", "right");
		defaultGamePadConfigMap.put("z", "rotateLeft");
		defaultGamePadConfigMap.put("-z", "rotateRight");
		defaultGamePadConfigMap.put("1", "force");
		defaultGamePadConfigMap.put("0", "pass");
		defaultGamePadConfigMap.put("2", "chipKick");
		defaultGamePadConfigMap.put("5", "dribble");
		defaultGamePadConfigMap.put("4", "arm");
		
		defaultKeyboardConfigMap.put("W", "forward");
		defaultKeyboardConfigMap.put("S", "backward");
		defaultKeyboardConfigMap.put("A", "left");
		defaultKeyboardConfigMap.put("D", "right");
		defaultKeyboardConfigMap.put("Q", "rotateLeft");
		defaultKeyboardConfigMap.put("E", "rotateRight");
		defaultKeyboardConfigMap.put("L", "force");
		defaultKeyboardConfigMap.put("K", "pass");
		defaultKeyboardConfigMap.put("J", "chipKick");
		defaultKeyboardConfigMap.put("U", "dribble");
		defaultKeyboardConfigMap.put("O", "arm");
	}
	

	public void saveConfig(File file, HashMap<String, String> map)
	{
		try
		{
			if (!file.exists())
			{
				file.createNewFile();
				ObjectOutputStream output = new ObjectOutputStream(new FileOutputStream(file));
				output.writeObject(map);
				output.flush();
				output.close();
			}
			
		} catch (IOException e)
		{
			e.printStackTrace();
		}
		log.info("File " + file + " saved.");
	}
	

	@SuppressWarnings("unchecked")
	public HashMap<String, String> loadConfig(File file)
	{
		try
		{
			ObjectInputStream input = new ObjectInputStream(new FileInputStream(file));
			HashMap<String, String> map = (HashMap<String, String>) input.readObject();
			return map;
		} catch (FileNotFoundException e)
		{
			e.printStackTrace();
		} catch (IOException e)
		{
			e.printStackTrace();
		} catch (ClassNotFoundException e)
		{
			e.printStackTrace();
		}
		return null;
	}
	

	public HashMap<String, String> loadDefaultConfig(AControllerPresenter newControllerPresenter)
	{
		AControllerPresenter controllerPresenter = newControllerPresenter;
		if (controllerPresenter.getType() == EControllerType.GAMEPAD)
			return defaultGamePadConfigMap;
		if (controllerPresenter.getType() == EControllerType.KEYBOARD)
			return defaultKeyboardConfigMap;
		return null;
	}
}
