/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - RCU
 * Date: 20.10.2010
 * Author(s): Lukas
 * 
 * *********************************************************
 */

package edu.dhbw.mannheim.tigers.robotcontrolutility.model;

import java.util.HashMap;

import net.java.games.input.Component;


/**
 * This class turns single componentactions of the controller into a HashMap with all
 * componentactions specified in the currentConfig.
 * 
 * @author Lukas
 * 
 */
public class ControllerInterpreter
{
	
	// --------------------------------------------------------------------------
	// --- instance variables ---------------------------------------------------
	// --------------------------------------------------------------------------
	private HashMap<String, String>	currentConfig;
	private ActionTranslator			translator;
	private HashMap<String, Double>	newActionMap	= new HashMap<String, Double>();
	
	
	// --------------------------------------------------------------------------
	// --- constructor(s) -------------------------------------------------------
	// --------------------------------------------------------------------------
	public ControllerInterpreter()
	{
		translator = new ActionTranslator();
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	public void setCurrentConfig(HashMap<String, String> currentConfig)
	{
		this.currentConfig = currentConfig;
	}
	

	/**
	 * add componentID to newActionMap if contained in currentConfig
	 */
	public void interpret(Component comp)
	{
		if (currentConfig.containsKey(comp.getIdentifier().toString()))
		{
			String command = currentConfig.get(comp.getIdentifier().toString());
			double value = comp.getPollData();
			newActionMap.put(command, value);
			translateCommandMap();
		}
	}
	

	private void translateCommandMap()
	{
		if (newActionMap.size() == currentConfig.size())
		{
			translator.translate(newActionMap);
			newActionMap.clear();
		}
	}
	

	/**
	 * Add all componentIDs with value 0.0 that are contained in currentConfig but not yet in newActionMap.
	 */
	public void fillActionMapWithNulls()
	{
		for (String cmd : currentConfig.values())
		{
			if (!newActionMap.containsKey(cmd))
			{
				newActionMap.put(cmd, 0d);
			}
		}
		translateCommandMap();
	}
	

	public boolean start(String serverName, int port)
	{
		return translator.start(serverName, port);
	}
	

	public void stop()
	{
		translator.stop();
	}
}
