/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - RCU
 * Date: 20.10.2010
 * Author(s): Lukas
 * 
 * *********************************************************
 */

package edu.dhbw.mannheim.tigers.robotcontrolutility.model;

import net.java.games.input.*;


/**
 * This class contains a Component.POV modifies its behavior as if this POV would be one of 8 Buttons.
 * 
 * @author Lukas
 * 
 */
public class POVToButton implements Component
{
	// --------------------------------------------------------------------------
	// --- instance variables ---------------------------------------------------
	// --------------------------------------------------------------------------
	Component	pov	= null;
	float			value	= 0;
	
	
	// --------------------------------------------------------------------------
	// --- constructor(s) -------------------------------------------------------
	// --------------------------------------------------------------------------
	public POVToButton(Component comp, float value)
	{
		this.pov = comp;
		this.value = value;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public float getDeadZone()
	{
		return 0;
	}
	

	@Override
	public Identifier getIdentifier()
	{
		if (pov.getIdentifier() == Component.Identifier.Axis.POV)
		{
			if (value == 0.125)
			{
				return new Identifier.Button("povNW");
			} else if (value == 0.25)
			{
				return new Identifier.Button("povN");
			} else if (value == 0.375)
			{
				return new Identifier.Button("povNE");
			} else if (value == 0.5)
			{
				return new Identifier.Button("povE");
			} else if (value == 0.625)
			{
				return new Identifier.Button("povSE");
			} else if (value == 0.75)
			{
				return new Identifier.Button("povS");
			} else if (value == 0.875)
			{
				return new Identifier.Button("povSW");
			} else if (value == 1.00)
			{
				return new Identifier.Button("povW");
			} else
			{
				System.out.println("Invalid Value for POV (POV to Button)");
			}
		}
		return pov.getIdentifier();
	}
	

	@Override
	public String getName()
	{
		if (pov.getIdentifier() == Component.Identifier.Axis.POV)
		{
			if (value == 0.125)
			{
				return "NorthWest";
			} else if (value == 0.25)
			{
				return "North";
			} else if (value == 0.375)
			{
				return "NorthEast";
			} else if (value == 0.5)
			{
				return "East";
			} else if (value == 0.625)
			{
				return "SouthEast";
			} else if (value == 0.75)
			{
				return "South";
			} else if (value == 0.875)
			{
				return "SouthWest";
			} else if (value == 1.00)
			{
				return "West";
			} else
			{
				System.out.println("Invalid Value for POV (POV to Button)");
			}
		}
		return null;
	}
	

	@Override
	public float getPollData()
	{
		if (pov.getIdentifier() == Component.Identifier.Axis.POV)
		{
			return ((pov.getPollData() == value) ? 1f : 0f);
		}
		return 0;
	}
	

	@Override
	public boolean isAnalog()
	{
		return false;
	}
	

	@Override
	public boolean isRelative()
	{
		if (pov.getIdentifier() == Component.Identifier.Axis.POV)
		{
			return pov.isRelative();
		}
		return false;
	}
	
}
