/*
 * *********************************************************
 * Copyright (c) 2009 DHBW Mannheim - Tigers Mannheim
 * Project: tigers-robotControlUtility
 * Date: 19.11.2010
 * Authors: Clemens Teichmann <clteich@gmx.de>
 * *********************************************************
 */
package edu.dhbw.mannheim.tigers.robotcontrolutility.presenter;

import java.io.File;
import java.util.HashMap;

import javax.swing.JTextField;

import net.java.games.input.Component;
import net.java.games.input.Controller;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.robotcontrolutility.model.ConfigManager;
import edu.dhbw.mannheim.tigers.robotcontrolutility.model.ControllerReader;
import edu.dhbw.mannheim.tigers.robotcontrolutility.view.ControllerPanel;
import edu.dhbw.mannheim.tigers.robotcontrolutility.view.MainFrame;


public abstract class AControllerPresenter
{
	// --------------------------------------------------------------------------
	// --- class variables ------------------------------------------------------
	// --------------------------------------------------------------------------
	private HashMap<String, String>	currentConfig	= new HashMap<String, String>();
	private Controller					controller;
	private ControllerPanel				cPanel;
	private ControllerReader			gPReader;
	
	private String							serverName;
	private int								portNumber;
	
	// --- Enumeration with controller types ---
	public enum EControllerType
	{
		GAMEPAD, KEYBOARD;
	}
	
	// --- logger ---
	private Logger	log	= Logger.getLogger(RCUPresenter.class.getName());
	
	
	// --------------------------------------------------------------------------
	// --- constructor(s) -------------------------------------------------------
	// --------------------------------------------------------------------------
	public AControllerPresenter(Controller newController)
	{
		this.controller = newController;
		log.info("Controller " + controller.getName() + " found");
		cPanel = MainFrame.getInstance().addControllerPanel(controller, AControllerPresenter.this);
		loadDefaultConfig();
		gPReader = new ControllerReader(controller);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @return Controller
	 */
	public Controller getController()
	{
		return controller;
	}
	

	/**
	 * recognize when new component selected. changes component to string and calls setCurrentConfig
	 * @param newComponent
	 * @param action
	 */
	public void onNewSelectedButton(Component newComponent, String action)
	{
		this.setCurrentConfig(newComponent.getIdentifier().toString(), action);
	}
	

	/**
	 * sets current config on new component
	 * @param key (Component)
	 * @param value (Action)
	 */
	public void setCurrentConfig(String key, String value)
	{
		// --- get textfield map with all actions (key) and textfields (value)
		HashMap<String, JTextField> tfMap = cPanel.getTextFieldMap();
		
		// --- iterate trough components ---
		for (Object ccKey : currentConfig.keySet())
		{
			// --- if currentConfig contains action - should always be entered ---
			if (value.equals(currentConfig.get(ccKey)))
			{
				// --- remove old entry ---
				currentConfig.remove(ccKey);
				
				// --- if currentConfig contains component ---
				if (currentConfig.containsKey(key))
				{
					// --- swap components ---
					currentConfig.put(ccKey.toString(), currentConfig.get(key));
					// --- swap textfield text ---
					tfMap.get(currentConfig.get(key)).setText(ccKey.toString());
				}
				// --- put new component and action in current configuration ---
				currentConfig.put(key, value);
				break;
			}
		}
		System.out.println("HashMapConfig: " + currentConfig.toString());
	}
	

	/**
	 * invokes saveConfig in ConfigManager
	 * @param file
	 */
	public void saveCurrentConfig(File file)
	{
		ConfigManager.getInstance().saveConfig(file, currentConfig);
	}
	

	/**
	 * 
	 * @param file
	 */
	public void loadCurrentConfig(File file)
	{
		// --- temporary HashMap - equals to currentConfig ---
		HashMap<String, String> tempMap = new HashMap<String, String>();
		
		// --- get config from file ---
		currentConfig = ConfigManager.getInstance().loadConfig(file);
		
		// --- iterate trough components ---
		for (Object key : currentConfig.keySet())
		{
			// --- iterate trough components of controller ---
			for (Component comp : controller.getComponents())
			{
				// --- check for negative components (axis)
				if (key.toString().substring(0, 1).equals("-"))
				{
					if (key.toString().substring(1).equals(comp.getIdentifier().toString()))
					{
						tempMap.put(key.toString(), currentConfig.get(key));
						break;
					}
				}
				if(key.toString().length() > 2)
				{
					if (key.toString().substring(0, 3).equals(comp.getIdentifier().toString()))
					{
						tempMap.put(key.toString(), currentConfig.get(key));
						System.out.println(tempMap);
						break;
					}
				}
				// --- check for all other components ---
				else
				{
					// --- if component found on controller, save in tempMap ---
					if (key.toString().equals(comp.getIdentifier().toString()))
					{
						tempMap.put(key.toString(), currentConfig.get(key));
						break;
					}
				}
			}
		}
		// --- eleven actions must be saved in config ---
		if (tempMap.size() == 11)
		{
			log.info("Succesfully loaded " + file.getName());
			cPanel.showConfig(tempMap);
		}
		// --- if any component not on the controller ---
		else
		{
			log.error("load configuration " + file.getName() + " failed \n configuration not compatible for controller");
		}
	}
	

	/**
	 * load Default Config on startUp
	 */
	public void loadDefaultConfig()
	{
		currentConfig = ConfigManager.getInstance().loadDefaultConfig(AControllerPresenter.this);
		cPanel.showConfig(currentConfig);
	}
	

	/**
	 * starts polling for controller
	 * @return activation status
	 */
	public boolean startPolling()
	{
		this.setConnectionInformation();
		gPReader.setCurrentConfig(currentConfig);
		return gPReader.startPolling(serverName, portNumber);
	}
	

	/**
	 * stops polling for controller
	 */
	public void stopPolling()
	{
		gPReader.stopPolling();
	}
	

	/**
	 * set server address and portnumber
	 */
	public void setConnectionInformation()
	{
		serverName = cPanel.getServerAdress();
		portNumber = cPanel.getPortNumber();
	}
	

	/**
	 * @return EControllerType
	 */
	public abstract EControllerType getType();
	
}
