/*
 * *********************************************************
 * Copyright (c) 2009 - 2010, DHBW Mannheim - Tigers Mannheim
 * Project: TIGERS - RCU
 * Date: 20.10.2010
 * Author(s): Lukas
 * 
 * *********************************************************
 */

package edu.dhbw.mannheim.tigers.robotcontrolutility.view.actions;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.robotcontrolutility.presenter.AControllerPresenter;
import edu.dhbw.mannheim.tigers.robotcontrolutility.presenter.RCUPresenter;
import edu.dhbw.mannheim.tigers.robotcontrolutility.view.ControllerPanel;
import edu.dhbw.mannheim.tigers.robotcontrolutility.view.utils.ConfFileFilter;


/**
 * This action saves the current config in the selected file.
 * 
 * @author Lukas
 *
 */

public class SaveConfigAction implements ActionListener
{
	// --------------------------------------------------------------------------
	// --- class variables ------------------------------------------------------
	// --------------------------------------------------------------------------
	private ControllerPanel			panel;
	private AControllerPresenter	cPresenter;
	private JFileChooser				fc				= new JFileChooser();
	private ConfFileFilter			confFilter	= new ConfFileFilter();
	private final File				DIR			= new File("configs");
	// --- logger ---
	private Logger						log		= Logger.getLogger(RCUPresenter.class.getName());
	
	
	// -----------------------------------------------------------------
	// ----- Constructor -----------------------------------------------
	// -----------------------------------------------------------------
	public SaveConfigAction(ControllerPanel panel, AControllerPresenter cPresenter)
	{
		this.panel = panel;
		this.cPresenter = cPresenter;
		fc.setFileFilter(confFilter);
		fc.setCurrentDirectory(DIR);
	}
	

	// -----------------------------------------------------------------
	// ----- Methods ---------------------------------------------------
	// -----------------------------------------------------------------
	public void actionPerformed(ActionEvent arg0)
	{
		int returnVal = fc.showSaveDialog(panel);
		if (returnVal == JFileChooser.APPROVE_OPTION)
		{
			File file = fc.getSelectedFile();
			if (!file.getName().endsWith(confFilter.getFileSuffix()))
			{
				file = new File(file.getAbsolutePath() + confFilter.getFileSuffix());
			}
			if (!file.exists())
			{
				log.info("Config saved in \""+file.getAbsolutePath()+"\"");
				cPresenter.saveCurrentConfig(file);
			} else
			{
				int answer = JOptionPane.showConfirmDialog(panel, "Overwrite " + file.getName() + "?");
				if (answer == JOptionPane.YES_OPTION)
				{
					log.info("Config saved in \""+file.getAbsolutePath()+"\"");
					cPresenter.saveCurrentConfig(file);
				}
				if (answer == JOptionPane.NO_OPTION)
				{
					this.actionPerformed(arg0);
				}
			}
		}
	}
}
