package edu.dhbw.mannheim.tigers.sim.engine;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;


/**
 * Provides control over the {@link SimEngine}s state to the inner ({@link ICore}) and outer ({@link IInputController} /
 * {@link ISimEngine}) parts of the application.
 * 
 * @author Gero
 */
public interface IBasicSimEngine {
	
	/**
	 * Sets up and start a new simulation
	 * 
	 * @see ESimEngineState#RUNNING
	 */
	public void start();
	

	/**
	 * Stops the whole running simulation, <b>including</b> InputHandling
	 * @see ESimEngineState#PAUSED_FULL
	 */
	public void pause();
	

	/**
	 * Pauses the physical part of a simulation (actual simulation is still running, e.g. InputHandling);
	 * could be resume()d.
	 * @see ESimEngineState#PAUSED_SIMULATION
	 */
	public void pauseSimulation();
	

	/**
	 * Resume a formerly pause()d simulation
	 * @see ESimEngineState#RUNNING
	 */
	public void resume();
	

	/**
	 * Stops and close a running/paused simulation
	 * @see ESimEngineState#STOPPED
	 */
	public void stop();
}
