package edu.dhbw.mannheim.tigers.sim.engine;

import com.jme.renderer.Renderer;

/**
 * This interface encapsulates together with {@link IUpdateableSimEnginePart} and {@link IRenderableSimEnginePart} inner
 * parts of the simulation which are capable of directly influencing it
 * 
 * @author Gero
 * 
 */
public interface ISimEnginePart extends IUpdateableSimEnginePart {
	
	/**
	 * @see SimEngine#initSystem()
	 */
	public abstract void initSystem();
	

	/**
	 * @see SimEngine#initGame()
	 */
	public abstract void initSimulation();
	

	/**
	 * @see SimEngine#cleanup()
	 */
	public abstract void cleanup();
	

	/**
	 * (called in render-loop)
	 * 
	 * @see SimEngine#render(float)
	 */
	public abstract void debug(float interpolation);
	
	
	/**
	 * Called after the {@link Renderer} has performed. Needed for video capture, e.g.
	 * 
	 * @param interpolation
	 * @param renderer
	 */
	public abstract void afterRenderCallback(float interpolation, Renderer renderer);
}