package edu.dhbw.mannheim.tigers.sim.engine.com.bot.tiger;

import java.io.IOException;

import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ABotStatusTransmitter;
import edu.dhbw.mannheim.tigers.sim.engine.com.bot.ICommunicator;
import edu.dhbw.mannheim.tigers.sim.util.network.UnicastUDPTransceiver;
import edu.dhbw.mannheim.tigers.sumatra.model.modules.impls.botmanager.commands.ACommand;


/**
 * The {@link ABotStatusTransmitter}-implementation for the Tiger-bot
 * 
 * @see TigerBotCommunicator
 * @see TigerBotCommandReceiver
 * 
 * @author Gero
 */
public class TigerBotStatusTransmitter extends ABotStatusTransmitter<ACommand> {

	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private final ITigerBotStatusProvider	botStatusProvider;
	private final UnicastUDPTransceiver transceiver;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	  * @see TigerBotStatusTransmitter
	  * @param communicator
	  * @param outputStream
	  * @param botStatusProvider
	  */
	public TigerBotStatusTransmitter(ICommunicator<ACommand> communicator, UnicastUDPTransceiver transceiver, ITigerBotStatusProvider botStatusProvider) {
		super(communicator);
		
		this.botStatusProvider = botStatusProvider;
		this.transceiver = transceiver;
	}

	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public boolean writeOutput() throws IOException {
		tempStatusList.clear();
		botStatusProvider.getBotStatus(tempStatusList);
		
		for (ACommand status : tempStatusList) {
			if (!transceiver.send(status.getTransferData())) {
				return false;
			}
		}
		
		return true;
	}


	@Override
	protected void closeOutput() throws IOException {
		transceiver.cleanup();
	}


	@Override
	public void send(ACommand msg) throws IOException
	{
		transceiver.send(msg.getTransferData());
	}
}
