package edu.dhbw.mannheim.tigers.sim.engine.delegates;

import org.apache.log4j.Logger;

import com.jme.renderer.Renderer;

import edu.dhbw.mannheim.tigers.sim.engine.ICore;
import edu.dhbw.mannheim.tigers.sim.engine.ISimEnginePart;


/**
 * <p>
 * This is the base class for all SimDelegates. They should separate the various purposes within the simulation from
 * each other.
 * </p>
 * 
 * @author Gero
 */
public abstract class ASimDelegate implements ISimEnginePart {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Util
	protected final Logger	log	= Logger.getLogger(getClass());
	
	protected final ICore	core;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see ASimDelegate
	 * @param core
	 */
	public ASimDelegate(ICore core) {
		this.core = core;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void initSystem() {
	}
	

	@Override
	public void initSimulation() {
	}
	

	@Override
	public void update(float interpolation, float tpc, float cps) {
	}
	

	@Override
	public void cleanup() {
	}
	

	@Override
	public void debug(float interpolation) {
	}
	
	
	@Override
	public void afterRenderCallback(float interpolation, Renderer renderer) {
	}
}
