package edu.dhbw.mannheim.tigers.sim.engine.model;


import com.jme.math.FastMath;
import com.jme.scene.Node;
import com.jme.system.DisplaySystem;
import com.jmex.physics.impl.ode.OdePhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.resources.ITextureProvider;
import edu.dhbw.mannheim.tigers.sim.engine.resources.TextureProvider;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelException;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelResourceException;


public abstract class ASimNode extends Node {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long			serialVersionUID	= -4986652404829467704L;
	
	// Resource Provider
	protected final ITextureProvider	texProvider			= TextureProvider.getInstance();
	
	// Physics
	protected final OdePhysicsSpace	space;
	
	// System
	protected final DisplaySystem		display;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see ASimNode
	 * @param space
	 * @param display
	 * @param name
	 * @throws SimModelResourceException
	 * @throws SimModelException
	 */
	public ASimNode(OdePhysicsSpace space, DisplaySystem display, String name) throws SimModelResourceException,
			SimModelException {
		super(name);
		
		this.space = space;
		this.display = display;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * A method to add the real physical node.
	 * MUST be called manually!!! (e.g. from the constructor...)
	 * 
	 * @throws SimModelResourceException
	 * @throws SimModelException
	 */
	protected abstract void createPhysicalNode() throws SimModelResourceException, SimModelException;
	

	// --------------------------------------------------------------------------
	// --- support --------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * {@link FastMath#DEG_TO_RADD}
	 * @param degree Degrees
	 * @return 
	 */
	protected static float rad(float degree) {
		return degree * FastMath.DEG_TO_RAD; // the pure angle
	}
	/**
	 * {@link FastMath#DEG_TO_RADD}
	 * @param rad Radians
	 * @return Degree
	 */
	protected static float deg(float radians) {
		return radians * FastMath.RAD_TO_DEG;
	}
}
