package edu.dhbw.mannheim.tigers.sim.engine.model;


import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.mmToReal;

import java.io.Serializable;

import com.jme.math.Quaternion;
import com.jme.math.Vector3f;


/**
 * A data-structure that associates a position and orientation with a timestamp
 * 
 * @author Gero
 */
public class TimedModelLocation implements Serializable {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long					serialVersionUID		= -4658128102368249422L;
	
	public static final TimedModelLocation	EMPTY_MODEL_LOCATION	= new TimedModelLocation(Vector3f.ZERO, 0, 0);
	
	private final Vector3f						position;
	private float									orientation;
	
	private long									timestamp;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * Does <strong>NOT</strong> copy position!
	 * 
	 * @param position
	 * @param orientation
	 * @param timestamp
	 */
	public TimedModelLocation(Vector3f position, float orientation, long timestamp) {
		this.position = position;
		this.orientation = orientation;
		this.timestamp = timestamp;
	}
	

	/**
	 * Copy-Constructor
	 * @param original
	 */
	public TimedModelLocation(TimedModelLocation original) {
		this.position = new Vector3f(original.position.clone());
		this.orientation = original.orientation;
		this.timestamp = original.timestamp;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	public void set(TimedModelLocation original) {
		this.set(original.getPosition(), original.getOrientation(), original.getTimestamp());
	}
	

	public void set(Vector3f position, float orientation, long timestamp) {
		this.position.set(position);
		this.orientation = orientation;
		this.timestamp = timestamp;
	}
	
	
	private static final float[]	floatDump	= new float[3];
	
	
	public TimedModelLocation(Vector3f worldTranslation, Quaternion worldRotation, long timestamp) {
		this(new Vector3f(worldTranslation), worldRotation.toAngles(floatDump)[1], timestamp);
	}
	

	public void set(Vector3f worldTranslation, Quaternion worldRotation, long timestamp) {
		set(worldTranslation, worldRotation.toAngles(floatDump)[1], timestamp);
	}
	

	public TimedModelLocation clone() {
		return new TimedModelLocation(this);
	}
	

	@Override
	public String toString() {
		return super.toString() + ": [ Timestamp: " + timestamp + ", Position: " + position + ", Orientation: "
				+ orientation + "]";
	}
	

	// --------------------------------------------------------------------------
	// --- getter ---------------------------------------------------------------
	// --------------------------------------------------------------------------
	public Vector3f getPosition() {
		return position;
	}
	

	public long getTimestamp() {
		return timestamp;
	}
	

	public float getOrientation() {
		return orientation;
	}
	

	/**
	 * @see UnitConversion#mmToReal(float)
	 */
	public float getXmm() {
		return mmToReal(position.x);
	}
	

	/**
	 * @see UnitConversion#mmToReal(float)
	 */
	public float getYmm() {
		return mmToReal(position.y);
	}
	

	/**
	 * @see UnitConversion#mmToReal(float)
	 */
	public float getZmm() {
		return mmToReal(position.z);
	}
}
