package edu.dhbw.mannheim.tigers.sim.engine.model.bot;


import com.jme.renderer.ColorRGBA;
import com.jme.system.DisplaySystem;
import com.jmex.physics.PhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.ICore;
import edu.dhbw.mannheim.tigers.sim.engine.com.ERoboCupTeam;


/**
 * The interface for a BotModelFactory
 * 
 * @author Gero
 */
public interface IBotModelFactory {
	

	/**
	 * Sets the {@link ICore} for this factory. Needed for bot <-> ball interaction.
	 * 
	 * @param newCore
	 */
	public void setCore(ICore newCore);
	

	/**
	 * Calls {@link #createBotModel(PhysicsSpace, DisplaySystem, String, ColorRGBA)} with <code>color == null</code>.
	 * 
	 * @param space
	 * @param display
	 * @param modelType
	 * @param team
	 * @return
	 */
	public abstract ABotModel createBotModel(PhysicsSpace space, DisplaySystem display, EBotModelType modelType,
			ERoboCupTeam team);
	

	/**
	 * Calls {@link #createBotModel(PhysicsSpace, DisplaySystem, String, ColorRGBA)}.
	 * 
	 * @param space
	 * @param display
	 * @param modelType
	 * @param team
	 * @param color
	 * @return
	 */
	public abstract ABotModel createBotModel(PhysicsSpace space, DisplaySystem display, EBotModelType modelType,
			ERoboCupTeam team, ColorRGBA color);
	

	/**
	 * Creates a {@link ABotModel} of the given type and color! =)
	 * If <code>modelType</code> doesn't represent a valid model type, {@link EBotModelType#CT_MODEL} is taken.
	 * 
	 * @param space
	 * @param display
	 * @param modelType
	 * @param team
	 * @param color
	 * @return
	 */
	public abstract ABotModel createBotModel(PhysicsSpace space, DisplaySystem display, String modelType,
			ERoboCupTeam team, ColorRGBA color);
	

	/**
	 * Calls {@link #createBotModel(PhysicsSpace, DisplaySystem, String, ColorRGBA)} with <code>color == null</code>.
	 * 
	 * @param space
	 * @param display
	 * @param modelType
	 * @param team
	 * @return
	 */
	public abstract ABotModel createBotModel(PhysicsSpace space, DisplaySystem display, String modelType,
			ERoboCupTeam team);
}