package edu.dhbw.mannheim.tigers.sim.engine.model.bot.parts;

import java.util.List;

import com.jme.math.Vector3f;
import com.jme.system.DisplaySystem;
import com.jmex.physics.CollisionGroup;
import com.jmex.physics.DynamicPhysicsNode;
import com.jmex.physics.PhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ABotModel;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBotPartsOwner;


/**
 * This abstract base class should help separate the various parts of a {@link ABotModel} from each other.
 * 
 * @see IBotPartsOwner
 * @author Gero
 */
public abstract class ABotModelPart implements IBotModelPart {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	protected final PhysicsSpace			space;
	protected final DisplaySystem			display;
	
	protected final IBotPartsOwner		owner;
	protected final DynamicPhysicsNode	body;
	/** Center of Mass (relative to the center of {@link #body}) */
	protected final Vector3f				com;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see ABotModelPart
	 * @param owner
	 * @param display
	 * @param body
	 */
	public ABotModelPart(IBotPartsOwner owner, DisplaySystem display, DynamicPhysicsNode body, Vector3f com) {
		this.space = body.getSpace();
		this.display = display;
		this.body = body;
		this.owner = owner;
		this.com = com;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @return Every {@link DynamicPhysicsNode} that should be added to the {@link ABotModel}s {@link CollisionGroup}
	 */
	public abstract List<DynamicPhysicsNode> getPhysicNodes();
}
