package edu.dhbw.mannheim.tigers.sim.engine.model.bot.tiger;

import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBotControl;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.parts.KickMode;


/**
 * This interface allows one the control over the {@link TigerBotModel}s movement and
 * kicking-device.
 * 
 * @author Gero
 */
public interface ITigerBotControl extends IBotControl {
	
	/**
	 * @return The current {@link TigerBotStatus}
	 */
	public abstract TigerBotStatus getBotStatus();
	
	
	/**
	 * @return A reference to this bots {@link ITigerMotorControl}
	 */
	public abstract ITigerMotorControl getMotorControl();
	

	/**
	 * @param percent
	 * @param mode
	 */
	public abstract void kickStraight(int percent, KickMode mode);
	

	/**
	 * Whether the capacitor of the bot should be charged or not
	 * 
	 * @param charge
	 */
	public abstract void charge(boolean charge);
	

	/**
	 * @param rpm
	 */
	public abstract void dribble(int rpm);
	

	/**
	 * @param percent
	 * @param mode
	 */
	public abstract void lob(int percent, KickMode mode);
	
	
	/**
	 * @return The CPU-ID that allows unique identification of each Tiger-bot (24 digits hex)
	 */
	public abstract String getCpuId();
}
