package edu.dhbw.mannheim.tigers.sim.engine.model.field;


import com.jme.scene.Node;

import edu.dhbw.mannheim.tigers.sim.engine.model.data.field.IRoboCupFieldData;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelException;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelResourceException;

/**
 * <p>
 * This represents a simple RoboCup field in the Sim.Model.
 * It consists of the fields ground, the goals and the surrounding border.
 * </p>
 * <p>
 * The null-vector lies in the center (and top) of the field, with its X-axis representing
 * the shorter side of the field, Z pointing to goal "A" and Y - starting at the top of
 * the field - pointing at the sun.=)
 * </p>
 * <p>
 * Side "A" is the positive one along the Z-axis, "B" negative; P is positive along X-axis,
 * N negative.
 * </p>
 * 
 * @author Gero
 */
public class RoboCupField extends Node {

	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long serialVersionUID = -3280791492325647538L;

	// Geometry
	private IRoboCupFieldData data;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public RoboCupField(IRoboCupFieldData data) throws SimModelResourceException, SimModelException {
		super("RoboCupField");
		this.data = data;
	}
	
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	public IRoboCupFieldData getData() {
		return data;
	}
}
