package edu.dhbw.mannheim.tigers.sim.engine.modes.test;

import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.cm;
import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.data;
import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.m;

import java.util.ArrayList;
import java.util.List;

import com.jme.input.InputHandler;
import com.jme.input.KeyInput;
import com.jme.input.action.InputAction;
import com.jme.input.action.InputActionEvent;
import com.jme.math.Vector3f;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;
import com.jmex.physics.DynamicPhysicsNode;
import com.jmex.physics.impl.ode.OdePhysicsSpace;

import edu.dhbw.mannheim.tigers.sim.engine.ASimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.ESimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.SimEngine;
import edu.dhbw.mannheim.tigers.sim.engine.com.ERoboCupTeam;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.ASimDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.BasicInputDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.BasicInputDelegate.InputType;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.BotLabelDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.ControlBotDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.FreeCameraDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.MousePickingDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.ScreenshotDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.SimPhysicsDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.SteerableBall;
import edu.dhbw.mannheim.tigers.sim.engine.delegates.VideoCaptureDelegate;
import edu.dhbw.mannheim.tigers.sim.engine.model.ModelDataProvider;
import edu.dhbw.mannheim.tigers.sim.engine.model.ModelHelper;
import edu.dhbw.mannheim.tigers.sim.engine.model.ball.IBallModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.model.ball.RoboCupBall;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.ABotModel;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.EBotModelType;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.IBasicBotModelControl;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.parts.KickMode;
import edu.dhbw.mannheim.tigers.sim.engine.model.bot.tiger.ITigerBotControl;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ASimEngineMode;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ESimEngineMode;
import edu.dhbw.mannheim.tigers.sim.engine.modes.IMatchControlProvider;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelException;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimModelResourceException;
import edu.dhbw.mannheim.tigers.sim.settings.ISettingsProvider;
import edu.dhbw.mannheim.tigers.sim.settings.data.ITestModeProperties;


/**
 * This {@link ASimEngineMode}-implementation is for debugging/testing. Several new (or once-new) features are
 * implemented and tested here.
 * 
 * @author Gero
 */
public class TestMode extends ASimEngineMode implements IMatchControlProvider {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private final List<IBasicBotModelControl>	botControls	= new ArrayList<IBasicBotModelControl>();
	private IBallModelControl						ballControl;
	
	// Delegates
	private SimPhysicsDelegate						physicsDelegate;
	private FreeCameraDelegate						camDelegate;
	private BasicInputDelegate						basicInputDelegate;
	private SteerableBall							steerableBall;
	private MousePickingDelegate					mousePickingDelegate;
	private ScreenshotDelegate						screenshotDelegate;
	private ControlBotDelegate						controlBotDelegate;
	private BotLabelDelegate						botLabelDelegate;
	@SuppressWarnings("unused")	// Needs not update, though!
	private VideoCaptureDelegate					videoDelegate;
	

	// Properties
	private final ITestModeProperties			props;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	  * @see TestMode
	  * @param core
	  */
	public TestMode(SimEngine core) {
		super(core, ESimEngineMode.TEST);
		
		props = ISettingsProvider.INSTANCE.getSimProperties().getTestModeProperties();
	}
	

	// State Management ###################################
	@Override
	public void changeStateTo(ESimEngineState newState) {
		
		switch (newState) {
			case PAUSED_FULL:
				currentState = pausedFull;
				break;
			
			case PAUSED_SIMULATION:
				currentState = pauseSimulation;
				break;
			
			case RUNNING:
				currentState = runningSimulation;
				break;
		}
	}
	
	private ASimEngineState	pauseSimulation	= new ASimEngineState(ESimEngineState.PAUSED_SIMULATION) {
		public void update(float interpolation, float tpc, float cps) {
			
			camDelegate.update(interpolation, tpc, cps);
			basicInputDelegate.update(interpolation, tpc, cps);
			testmodeDelegate.update(interpolation, tpc, cps);
			mousePickingDelegate.update(interpolation, tpc, cps);
			screenshotDelegate.update(interpolation, tpc, cps);
			botLabelDelegate.update(interpolation, tpc, cps);
		};
	};
	
	private ASimEngineState	runningSimulation	= new ASimEngineState(ESimEngineState.RUNNING) {
		public void update(float interpolation, float tpc, float cps) {
			
			physicsDelegate.update(interpolation, tpc, cps);
			camDelegate.update(interpolation, tpc, cps);
			basicInputDelegate.update(interpolation, tpc, cps);
			testmodeDelegate.update(interpolation, tpc, cps);
			steerableBall.update(interpolation, tpc, cps);
			mousePickingDelegate.update(interpolation, tpc, cps);
			screenshotDelegate.update(interpolation, tpc, cps);
			controlBotDelegate.update(interpolation, tpc, cps);
			botLabelDelegate.update(interpolation, tpc, cps);
		};
	};
	
	
	@Override
	public void setupDelegates() {
		addDelegate(physicsDelegate = new SimPhysicsDelegate(core));
		addDelegate(camDelegate = new FreeCameraDelegate(core));
		addDelegate(testmodeDelegate);
		addDelegate(basicInputDelegate = new BasicInputDelegate(core, InputType.FIRST_PERSON));
		addDelegate(steerableBall = new SteerableBall(core));
		addDelegate(mousePickingDelegate = new MousePickingDelegate(core));
		addDelegate(screenshotDelegate = new ScreenshotDelegate(core));
		addDelegate(controlBotDelegate = new ControlBotDelegate(core));
		addDelegate(botLabelDelegate = new BotLabelDelegate(core, this));
		addDelegate(videoDelegate = new VideoCaptureDelegate(core));
	}
	
	
	private float			posB_Y				= m(0.5f);
	private ABotModel		bot1;
	private ASimDelegate	testmodeDelegate	= new ASimDelegate(core) {
															
		private ABotModel				tBot		= null;
		private ITigerBotControl	tControl	= null;
		
		
		public void initSimulation() {
			final OdePhysicsSpace space = (OdePhysicsSpace) core.getPhysicsSpace();
			
			// The field
			try {
				Node field = factory.createField(space, core.getDisplay(), ModelDataProvider.STAN_FIELD);
				core.getStatics().attachChild(field);
			} catch (SimModelResourceException exc) {
				log.error(exc.getMessage(), exc);
			} catch (SimModelException exc) {
				log.error(exc.getMessage(), exc);
			}
			
			// Test-Bots
			bot1 = factory.createBotModel(space, core.getDisplay(),
					props.getBotModelType(), ERoboCupTeam.YELLOW_TEAM);
			bot1.setLocation(new Vector3f(cm(30), data(bot1.getHeight()) / 2,
					cm(30)));
			
			// Quaternion q = new Quaternion();
			// q.fromAngleAxis((float) Math.toRadians(90), Vector3f.UNIT_X);
			// bot1.setRotation(q);
			
			core.getRobots().attachChild(bot1);
			bot1.setActive(true);
			
			botControls.add(bot1);
			// updateableModels.add(bot1);
			

			ABotModel bot2 = factory.createBotModel(space, core.getDisplay(),
					props.getBotModelType(), ERoboCupTeam.YELLOW_TEAM);
			bot2.setLocation(new Vector3f(-cm(30), data(bot2.getHeight()) / 2,
					-cm(30)));
			
			// Quaternion q2 = new Quaternion();
			// q2.fromAngleAxis((float) Math.toRadians(90), Vector3f.UNIT_Z);
			// bot2.setRotation(q2);
			
			core.getRobots().attachChild(bot2);
			bot2.setActive(true);
			
			botControls.add(bot2);
			updateableModels.add(bot2);
			
			ColorRGBA tColor = ColorRGBA.red.clone();
			tColor.a *= 0.3f;
			tBot = factory.createBotModel(space, core.getDisplay(),
					EBotModelType.TIGER_MODEL, ERoboCupTeam.YELLOW_TEAM, tColor);
			tBot.setLocation(new Vector3f(cm(30), data(tBot.getHeight()) / 2,
					-cm(30)));
			
			core.getRobots().attachChild(tBot);
			tBot.setActive(true);
			
			botControls.add(tBot);
			updateableModels.add(tBot);
			
			// First ball
			RoboCupBall firstBall = factory.createRoboCupBall(space, core.getDisplay());
			steerableBall.setBall(firstBall);
			core.getBalls().attachChild(firstBall);
			
			ballControl = firstBall;
			

			// Second ball
			DynamicPhysicsNode secondBall = factory.createRoboCupBall(space, core.getDisplay());
			secondBall.setLocalTranslation(new Vector3f(0, posB_Y, 0));
			
			ModelHelper.color(secondBall, core.getDisplay(), ColorRGBA.blue);
			
			core.getBalls().attachChild(secondBall);
			secondBall.setActive(true);
			

			tControl = (ITigerBotControl) tBot;
			core.getInputHandler().addAction(new InputAction() {
				@Override
				public void performAction(InputActionEvent evt) {
					
					if (!evt.getTriggerPressed()) { // Trigger released
						tControl.kickStraight(100, KickMode.FORCE);
					}
				}
				
			}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_U, InputHandler.AXIS_NONE, false);
			
			core.getInputHandler().addAction(new InputAction() {
				@Override
				public void performAction(InputActionEvent evt) {
					
					if (!evt.getTriggerPressed()) { // Trigger released
						tControl.lob(100, KickMode.FORCE);
					}
				}
				
			}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_I, InputHandler.AXIS_NONE, false);
			
			core.getInputHandler().addAction(new InputAction() {
				@Override
				public void performAction(InputActionEvent evt) {
					
					if (!evt.getTriggerPressed()) { // Trigger released
						tControl.kickStraight(100, KickMode.ARM);
					}
				}
				
			}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_L, InputHandler.AXIS_NONE, false);
			
			core.getInputHandler().addAction(new InputAction() {
				@Override
				public void performAction(InputActionEvent evt) {
					
					if (!evt.getTriggerPressed()) { // Trigger released
						tControl.dribble(10000);
					}
				}
				
			}, InputHandler.DEVICE_KEYBOARD, KeyInput.KEY_O, InputHandler.AXIS_NONE, false);
			
			// Yes, load capacitor!
			tControl.charge(true);
		};
	};
	
	
	// IBotControlProvider
	@Override
	public IBallModelControl getBallControl() {
		return ballControl;
	}
	

	@Override
	public List<? extends IBasicBotModelControl> getBotControls() {
		return botControls;
	}
}
