package edu.dhbw.mannheim.tigers.sim.engine.observer;


import java.util.concurrent.ExecutorService;

import edu.dhbw.mannheim.tigers.sim.engine.ESimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.SimEngine;


/**
 * Dispatches the state-events coming from the {@link SimEngine} and thereby guarantees a certain state for
 * every time due to synchronization ({@link #currentStateSync}).
 * 
 * @author Gero
 */
public abstract class AStatefulSimEngineObserver extends ASimEngineObserver {
   
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
   // State
   private final Object    currentStateSync = new Object();
   private ESimEngineState currentState     = ESimEngineState.STOPPED;
   private ESimEngineState lastState        = ESimEngineState.STOPPED;
   
   
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
   /**
     * @see AStatefulSimEngineObserver
     * @param newExService
     * @param oneThreadPerCall
     * @param name
     */
   public AStatefulSimEngineObserver(ExecutorService newExService, boolean oneThreadPerCall,
         String name) {
      super(newExService, oneThreadPerCall, name);
   }
   

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
   @Override
   protected void onUpdate(ISimEngineObservable observable, ESimEngineState event) {
      
      synchronized (currentStateSync) {
         lastState = currentState;
         currentState = event;
         
         switch (event) {
            
            case RUNNING:
               if (lastState.equals(ESimEngineState.PAUSED_FULL)
                     || lastState.equals(ESimEngineState.PAUSED_SIMULATION)) {
                  onResume();
               } else {
                  onStart();
               }
               break;
            
            case PAUSED_FULL:
            case PAUSED_SIMULATION:
               onPause();
               break;
            
            case STOPPED:
               observable.removeObserver(this);
               onStop();
               break;
            
            default:
               break;
         }
      }
   }
   

   /**
    * Called when the SimEngine has been resumed
    */
   protected abstract void onResume();
   

   /**
    * Called when the SimEngine has been started
    */
   protected abstract void onStart();
   

   /**
    * Called when the SimEngine has been paused
    */
   protected abstract void onPause();
   

   /**
    * Called when the SimEngine has been stopped
    */
   protected abstract void onStop();
   

   public ESimEngineState getCurrentState() {
      return this.currentState;
   }
}
