package edu.dhbw.mannheim.tigers.sim.engine.plugins;

import java.util.List;

import edu.dhbw.mannheim.tigers.sim.exceptions.SimPluginException;
import edu.dhbw.mannheim.tigers.sim.resources.IResourceProvider;


/**
 * This interface manages plugins - similar to an {@link IResourceProvider} handling resources. They are loaded,
 * unloaded and reloaded, one can ask for a list of loaded plugins and retrieve a special one.
 * 
 * @author Gero
 */
public interface IPluginProvider {
	
	public static final IPluginProvider	INSTANCE	= PluginProvider.getInstance();
	
	
	/**
	 * Tries to load all available plugins
	 */
	public void loadPlugins();
	

	/**
	 * Combined unload and {@link #loadPlugins()}
	 * 
	 * @throws SimPluginException
	 */
	public void reloadPlugins();
	

	/**
	 * Cleans up any internally used resources and all plugins
	 */
	public void cleanup();
	

	/**
	 * @param type
	 * @return A list of {@link PluginID} identifying the loaded plugins of the given type
	 */
	public List<PluginID> getLoadedPluginsOf(EPluginType type) throws SimPluginException;
	

	/**
	 * @return A list of all loaded plugins
	 */
	public List<ASimPlugin> getLoadedPlugins();
	

	/**
	 * @param <T>
	 * @param id
	 * @param type
	 * @return The plugin identified by given id and type. If there is no {@link ASimPlugin} associated with this
	 *         combination, or <code>id == null</code>, the default implementation for this {@link EPluginType}
	 * @throws SimPluginException
	 */
	public <T extends ASimPlugin> T getPlugin(PluginID id, EPluginType type);
	

	/**
	 * Persists the settings for every loaded plugin
	 */
	public void saveLoadedPluginsProperties();
}
