package edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.configuration.SubnodeConfiguration;

import com.jme.renderer.Renderer;
import com.jme.scene.Node;
import com.jme.system.DisplaySystem;

import edu.dhbw.mannheim.tigers.sim.engine.modes.IMatchControlProvider;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.EPluginType;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.PluginID;


/**
 * Default-implementation of the {@link AVisualizationPlugin}
 * 
 * @author Gero
 */
public class DefaultVisualizationPlugin extends AVisualizationPlugin {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	public static final PluginID	ID	= new PluginID("DEFAULT_VISUALIZATION");
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see DefaultVisualizationPlugin
	 * @param config
	 */
	public DefaultVisualizationPlugin(SubnodeConfiguration config) {
		super(config);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public List<IDrawer> getPoiDrawers(Node root, DisplaySystem display, IMatchControlProvider matchControl) {
		List<IDrawer> drawers = new ArrayList<IDrawer>();
		
		drawers.add(new DefaultBotDrawer(root, Renderer.QUEUE_TRANSPARENT, display, matchControl));
		drawers.add(new DefaultBallDrawer(root, Renderer.QUEUE_TRANSPARENT, display));
		drawers.add(new DefaultPathDrawer(root, Renderer.QUEUE_OPAQUE, display));
		drawers.add(new DefaultSituationsDrawer(root, Renderer.QUEUE_TRANSPARENT, display));
		
		return drawers;
	}
	
	
	@Override
	public void checkConfigConsistency(SubnodeConfiguration config) {
		// No properties yet
	}
	

	@Override
	public PluginID getID() {
		return ID;
	}
	

	@Override
	public EPluginType getType() {
		return EPluginType.VISUALIZATION;
	}
}
