package edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization;

import static edu.dhbw.mannheim.tigers.sim.engine.model.UnitConversion.cm;

import com.jme.bounding.BoundingSphere;
import com.jme.renderer.ColorRGBA;
import com.jme.scene.Node;
import com.jme.scene.shape.Sphere;
import com.jme.system.DisplaySystem;

import edu.dhbw.mannheim.tigers.sim.engine.model.ModelHelper;
import edu.dhbw.mannheim.tigers.sim.engine.plugins.visualization.SimVisMessages.SimVisData.Poi;


/**
 * Visual representation of a predicted ball position
 * 
 * @author Gero
 */
public class PredictedBall extends AVisualObject {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	// Constants
	private final ColorRGBA			color						= ColorRGBA.cyan.clone();
	private static final float		BALL_RADIUS				= cm(2);
	private static final float		BALL_HEIGHT_OFFSET	= BALL_RADIUS + cm(1.5f);
	
	private final DisplaySystem	display;
	
	// Visuals
	private final Sphere				sphere;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see PredictedBall
	 * @param poi
	 * @param parent
	 * @param display
	 */
	PredictedBall(Poi poi, Node parent, DisplaySystem display) {
		super(poi, parent);
		
		this.display = display;
		
		// Visuals
		sphere = new Sphere("Poi", 10, 10, BALL_RADIUS);
		sphere.setModelBound(new BoundingSphere());
		sphere.updateModelBound();
		
		localRoot.attachChild(sphere);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected AVisualObject drawSingle(Poi poi) {
		
		color.a = 0.6f;
		color.a *= poi.getValue() == 0f ? 1 : poi.getValue();
		ModelHelper.color(sphere, display, color);
		
		sphere.setLocalTranslation(x(poi.getPos()), BALL_HEIGHT_OFFSET, z(poi.getPos()));
		
		return this;
	}
	

	@Override
	protected void undraw() {
		sphere.removeFromParent();
	}
	

	@Override
	protected void updateRenderState() {
		sphere.updateRenderState();
	}
}
