package edu.dhbw.mannheim.tigers.sim.gui;


import org.apache.log4j.Appender;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.engine.ESimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ESimEngineMode;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;


/**
 * This is the core-interface any GUI has to implement. Every interaction with the controllers is
 * done via this interface.
 * 
 * @see SwingGUI
 * 
 * @author Gero
 */
public interface IGUI {
   
   /**
    * @param logAppender The instance of the {@link Appender} which holds the log-events
    *           fired during startup, before initialization of the GUI
    */
   public void intialize(Appender logAppender);
   

   /**
    * @param enabled Whether the GUI should be en- or disabled
    */
   public void setEnabled(boolean enabled);
   

   /**
    * @param newState The new {@link ESimEngineState} to set
    */
   public void refreshEngineState(ESimEngineState newState);
   

   /**
    * Closes the GUI
    */
   public void close();
   

   /**
    * @return The {@link ESimEngineMode} currently selected in the GUI
    */
   public ESimEngineMode getSelectedMode();
   

   /**
    * @return The currently set {@link IInputController}-implementation
    */
   public IInputController getInputController();
   

   /**
    * @return Whether the user wants the simulation to be stopped or not
    */
   public boolean verifyEndOfSimulation();
}
