package edu.dhbw.mannheim.tigers.sim.gui.swing;

import javax.swing.JComboBox;
import javax.swing.JToolBar;

import edu.dhbw.mannheim.tigers.sim.engine.ESimEngineState;
import edu.dhbw.mannheim.tigers.sim.engine.modes.ESimEngineMode;
import edu.dhbw.mannheim.tigers.sim.gui.swing.actions.SimPauseFullAction;
import edu.dhbw.mannheim.tigers.sim.gui.swing.actions.SimPauseSimulationAction;
import edu.dhbw.mannheim.tigers.sim.gui.swing.actions.SimStartResumeAction;
import edu.dhbw.mannheim.tigers.sim.gui.swing.actions.SimStopAction;


/**
 * This toolbar lets the user control the simulation
 * 
 * @see IGUIEngineControl
 * @author Gero
 * 
 */
public class SimControlToolbar extends JToolBar implements IGUIEngineControl {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long					serialVersionUID	= 8261313451930432670L;
	
	private final SimStartResumeAction		startResume;
	private final SimPauseSimulationAction	pauseSim;
	private final SimPauseFullAction			pauseFull;
	private final SimStopAction				stop;
	private final JComboBox						modeSwitch;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see SimControlToolbar
	 * @param gui
	 */
	public SimControlToolbar(SwingGUI gui) {
		super("SimControl");
		
		add(modeSwitch = new EngineModeSwitch());
		add(startResume = SimStartResumeAction.getInstance(gui));
		add(pauseSim = SimPauseSimulationAction.getInstance(gui));
		add(pauseFull = SimPauseFullAction.getInstance(gui));
		add(stop = SimStopAction.getInstance(gui));
		
		refresh(ESimEngineState.STOPPED);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void refresh(ESimEngineState newState) {
		
		switch (newState) {
			// TODO Dependency to sim.core.model => doof! =(
			case PAUSED_SIMULATION:
			case PAUSED_FULL:
				startResume.setEnabled(true);
				startResume.setRunning(true);
				
				pauseSim.setEnabled(false);
				
				pauseFull.setEnabled(false);
				
				stop.setEnabled(true);
				
				modeSwitch.setEnabled(false);
				break;
			
			case READY:
			case RUNNING:
				startResume.setEnabled(false);
				startResume.setRunning(false);
				
				pauseSim.setEnabled(true);
				
				pauseFull.setEnabled(true);
				
				stop.setEnabled(true);
				
				modeSwitch.setEnabled(false);
				break;
			
			case STOPPED:
				startResume.setEnabled(true);
				startResume.setRunning(false);
				
				pauseSim.setEnabled(false);
				
				pauseFull.setEnabled(false);
				
				stop.setEnabled(false);
				
				modeSwitch.setEnabled(true);
		}
	}
	

	@Override
	public ESimEngineMode getSelectedMode() {
		return (ESimEngineMode) modeSwitch.getSelectedItem();
	}
}
