package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;

import java.awt.IllegalComponentStateException;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Icon;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.gui.IGUI;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;


/**
 * Base class for every action of the {@link SwingGUI} {@link IGUI}-implementation
 * 
 * @author Gero
 */
public abstract class ASimAction extends AbstractAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long			serialVersionUID	= -8481498480420260878L;
	
	protected final Logger				log					= Logger.getLogger(getClass());
	
	protected final Object				actionSync			= new Object();
	protected final IInputController	input;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	  * @see ASimAction
	  * @param name
	  * @param description
	  * @param input
	  */
	protected ASimAction(String name, String description, IInputController input) {
		super(name);
		
		if (description != null && !description.isEmpty()) {
			this.putValue(SHORT_DESCRIPTION, description);
		}
		
		this.input = input;
	}
	

	/**
	  * @see ASimAction
	  * @param name
	  * @param description
	  * @param input
	  * @param icon
	  */
	protected ASimAction(String name, String description, IInputController input, Icon icon) {
		super(name, icon);
		
		this.putValue(SHORT_DESCRIPTION, description);
		
		this.input = input;
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void actionPerformed(final ActionEvent e) {
		try {
		
			synchronized (actionSync) {
				final String actionName = getValue("Name").toString();
				this.setEnabled(false);
				
				new Thread(new Runnable() {
					@Override
					public void run() {
						Thread.currentThread().setName("GUI: '" + actionName + "'");
						doPerformSyncAction(e);
					}
				}).start();
			}
		} catch (IllegalComponentStateException icse) {
			log.warn("GUI-error!");	// TODO Determine cause for this. See #434
		}
	}
	

	/**
	 * Implement your action here, but remember to enable action afterwards!!!
	 * @param event
	 */
	public abstract void doPerformSyncAction(ActionEvent event);
}
