package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;



/**
 * This action initializes the application shutdown
 * 
 * @author Gero
 */
public class ExitAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long serialVersionUID = 1L;

	private static ExitAction instance;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public synchronized static ExitAction getInstance(IInputController input) {
		if (instance == null) {
			EImage imgType = EImage.EXIT;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new ExitAction(input, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(ExitAction.class).debug("No image for '" + imgType + "'!",
						err);
				instance = new ExitAction(input);
			}
		}
		return instance;
	}
	
	
	private ExitAction(IInputController input) {
		super("Exit", "Exit the simulator", input);
	}
	
	
	private ExitAction(IInputController input, Icon icon) {
		super("Exit", "Exit the simulator", input, icon);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void doPerformSyncAction(ActionEvent event) {
		try {
			input.exitApp();
		} finally {
			setEnabled(true);		// In case someone doesn't want to exit
		}
	}
	
}
