package edu.dhbw.mannheim.tigers.sim.gui.swing.actions;

import java.awt.event.ActionEvent;

import javax.swing.Icon;

import org.apache.log4j.Logger;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.gui.swing.SwingGUI;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * Calls {@link IInputController#pauseSimulation()} which stops the simulation but not the engine (camera is still
 * moveable, for example)
 * 
 * @author Gero
 * 
 */
public class SimPauseSimulationAction extends ASimAction {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long						serialVersionUID	= -6912115025615925013L;
	
	private static final String					TITLE					= "Pause";
	private static final String					DESCRIPTION			= "This pauses the physical simulation only";
	
	private static SimPauseSimulationAction	instance;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	public synchronized static SimPauseSimulationAction getInstance(SwingGUI gui) {
		if (instance == null) {
			EImage imgType = EImage.CONTROL_PAUSE_SIMULATION;
			try {
				Icon icon = ImageProvider.getInstance().getIcon(imgType);
				instance = new SimPauseSimulationAction(gui, icon);
			} catch (SimResourceException err) {
				Logger.getLogger(SimPauseFullAction.class).debug("No image for '" + imgType + "'!", err);
				instance = new SimPauseSimulationAction(gui);
			}
		}
		return instance;
	}
	

	private SimPauseSimulationAction(SwingGUI gui) {
		super(TITLE, DESCRIPTION, gui.getInputController());
	}
	

	private SimPauseSimulationAction(SwingGUI gui, Icon icon) {
		super(TITLE, DESCRIPTION, gui.getInputController(), icon);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public void doPerformSyncAction(ActionEvent event) {
		
		input.pauseSimulation();
	}
}
