package edu.dhbw.mannheim.tigers.sim.gui.swing.settings;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.settings.ISettingsProvider;
import edu.dhbw.mannheim.tigers.sim.settings.data.ICamProperties;
import edu.dhbw.mannheim.tigers.sim.settings.data.IDataFactory;


/**
 * Displays the camera-settings
 * 
 * @author Gero
 */
public class CamSettingsPanel extends ASettingsPanel {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long	serialVersionUID	= -3233509049715548443L;

	private ICamProperties		properties;
	
	// Cam Properties
	private JPanel					camPropertiesPanel;
	
	private JTextField			textFPS;
	
	private JTextField			textMcastInterface;
	private JTextField			textPort;
	private JTextField			textMulticastAddress;
	
	private JComboBox				textCamFrameBuilder;

	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------	
	/**
	  * @see CamSettingsPanel
	  * @param inputController
	  */
	public CamSettingsPanel(IInputController inputController) {
		super("Cam", inputController);
	}
	
	
	@Override
	protected void loadSettings() {
		properties = ISettingsProvider.INSTANCE.getSimProperties().getCamProperties();
	}
	
	
	@Override
	protected void createGui() {
		camPropertiesPanel = createDefaultSettingsPanel("Cam Properties");
		
		camPropertiesPanel.add(new JLabel("FPS:"));
		camPropertiesPanel.add(textFPS = new JTextField());
		
		
		camPropertiesPanel.add(new JLabel("Multicast interface:"));
		camPropertiesPanel.add(textMcastInterface = new JTextField());
		
		camPropertiesPanel.add(new JLabel("Port:"));
		camPropertiesPanel.add(textPort = new JTextField());
		
		camPropertiesPanel.add(new JLabel("Multicast Address:"));
		camPropertiesPanel.add(textMulticastAddress = new JTextField());
		
		
		camPropertiesPanel.add(new JLabel("Cam frame builder:"));
		textCamFrameBuilder = new JComboBox();
		for (String p : inputController.getLoadedCamOutputPlugins()) {
			textCamFrameBuilder.addItem(p);
		}
		camPropertiesPanel.add(textCamFrameBuilder);
		add(camPropertiesPanel);
		
		addButtonsPanel();
	}
	
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public boolean checkInput() {
		// Nothing to do here
		return true;
	}
	
	
	@Override
	public void defaults() {
		fillFields(IDataFactory.eINSTANCE.createCamProperties());
	}
	

	private void fillFields(ICamProperties camProperties) {
		textFPS.setText(String.valueOf(camProperties.getFPS()));
		
		textMcastInterface.setText(camProperties.getMcastInterface());
		textPort.setText(String.valueOf(camProperties.getTargetPort()));
		textMulticastAddress.setText(String.valueOf(camProperties.getMcastAddress()));
		
		textCamFrameBuilder.setSelectedItem(camProperties.getPluginID());
	}
	

	@Override
	public void revert() {
		fillFields(properties);
	}
	

	@Override
	public void apply() {
		properties.setFPS(Integer.parseInt(textFPS.getText()));
		
		properties.setMcastInterface(textMcastInterface.getText());
		properties.setTargetPort(Integer.parseInt(textPort.getText()));
		properties.setMcastAddress(textMulticastAddress.getText());
		
		properties.setPluginID(String.valueOf(textCamFrameBuilder.getSelectedItem()));
	}
}
