package edu.dhbw.mannheim.tigers.sim.gui.swing.settings;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import edu.dhbw.mannheim.tigers.sim.controller.IInputController;
import edu.dhbw.mannheim.tigers.sim.settings.ISettingsProvider;
import edu.dhbw.mannheim.tigers.sim.settings.data.IDataFactory;
import edu.dhbw.mannheim.tigers.sim.settings.data.IVisualizationProperties;


/**
 * Displays the visualization-settings
 * 
 * @author Gero
 */
public class VisualizationSettingsPanel extends ASettingsPanel {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private static final long			serialVersionUID	= 3090762410247797250L;

	private IVisualizationProperties	properties;
	
	// Vis Properties
	private JPanel							visPropertiesPanel;
	
	private JTextField					textMulticastAddress;
	private JTextField					textPort;
	private JTextField					textMaxHoldTime;
	private JComboBox						visPlugin;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	  * @see VisualizationSettingsPanel
	  * @param inputController
	  */
	public VisualizationSettingsPanel(IInputController inputController) {
		super("Visualization", inputController);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void loadSettings() {
		properties = ISettingsProvider.INSTANCE.getSimProperties().getVisualizationProperties();
	}
	

	@Override
	protected void createGui() {
		visPropertiesPanel = createDefaultSettingsPanel("Visualization");
		
		visPropertiesPanel.add(new JLabel("Multicast Address:"));
		visPropertiesPanel.add(textMulticastAddress = new JTextField());
		
		visPropertiesPanel.add(new JLabel("Port:"));
		visPropertiesPanel.add(textPort = new JTextField());
		
		visPropertiesPanel.add(new JLabel("Maximum hold time:"));
		visPropertiesPanel.add(textMaxHoldTime = new JTextField());
		
		visPropertiesPanel.add(new JLabel("Visualization plugin:"));
		
		visPlugin = new JComboBox();
		for (String p : inputController.getLoadedVisualizationPlugins()) {
			visPlugin.addItem(p);
		}
		visPropertiesPanel.add(visPlugin);
		add(visPropertiesPanel);
		
		addButtonsPanel();
	}
	

	private void fillFields(IVisualizationProperties visProperties) {
		textMulticastAddress.setText(String.valueOf(visProperties.getAddress()));
		textPort.setText(String.valueOf(visProperties.getPort()));
		textMaxHoldTime.setText(String.valueOf(visProperties.getMaxHoldTime()));
		visPlugin.setSelectedItem(visProperties.getPluginID());
	}
	

	@Override
	public boolean checkInput() {
		return true;
	}
	

	@Override
	public void apply() {
		properties.setPluginID(String.valueOf(visPlugin.getSelectedItem()));
		properties.setPort(Integer.parseInt(textPort.getText()));
		properties.setMaxHoldTime(Integer.valueOf(textMaxHoldTime.getText()));
		properties.setAddress(textMulticastAddress.getText());
	}
	

	@Override
	public void revert() {
		fillFields(properties);
	}
	

	@Override
	public void defaults() {
		fillFields(IDataFactory.eINSTANCE.createVisualizationProperties());
	}
}
