package edu.dhbw.mannheim.tigers.sim.resources;


import java.awt.Image;
import java.util.HashMap;

import javax.swing.ImageIcon;

import com.jme.util.TextureManager;

import edu.dhbw.mannheim.tigers.sim.exceptions.SimResourceException;
import edu.dhbw.mannheim.tigers.sim.resources.ImageProvider.EImage;


/**
 * Provides a set of {@link ImageIcon}s / {@link Image}s / {@link com.jme.image.Image}s, mainly for the GUI
 * 
 * @author Gero
 */
public class ImageProvider extends AResourceProvider<EImage, ImageIcon> implements IImageProvider {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	public enum EImage {
		CONTROL_START,
		CONTROL_PAUSE_SIMULATION,
		CONTROL_PAUSE,
		CONTROL_STOP,
		MODEL_RELOAD,
		PLUGIN_GO,
		PLUGIN_EDIT,
		TIGER_ICON_32x32,
		TIGER_ICON_16x16,
		BULLET_WRENCH,
		EXIT;
	}
	
	public static final String		IMAGE_PATH	= "resources/images/";

	private static ImageProvider	instance		= null;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------	
	/**
	  * @see ImageProvider
	  */
	private ImageProvider() {
		super(ImageProvider.class);
	}
	

	public static synchronized ImageProvider getInstance() {
		if (instance == null) {
			instance = new ImageProvider();
		}
		return instance;
	}
	
	
	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	protected void doInitialize() {
		storage = new HashMap<EImage, ImageIcon>();
	}
	

	@Override
	protected void doLoad() {
		
		storage.put(EImage.CONTROL_START, new ImageIcon(IMAGE_PATH + "control_play.png"));
		
		storage.put(EImage.CONTROL_PAUSE_SIMULATION, new ImageIcon(IMAGE_PATH + "control_pause.png"));
		
		storage.put(EImage.CONTROL_PAUSE, new ImageIcon(IMAGE_PATH + "control_pause_blue.png"));
		
		storage.put(EImage.CONTROL_STOP, new ImageIcon(IMAGE_PATH + "control_stop.png"));
		
		storage.put(EImage.MODEL_RELOAD, new ImageIcon(IMAGE_PATH + "model_reload.png"));
		
		storage.put(EImage.PLUGIN_GO, new ImageIcon(IMAGE_PATH + "plugin_go.png"));
		
		storage.put(EImage.PLUGIN_EDIT, new ImageIcon(IMAGE_PATH + "plugin_edit.png"));
		
		storage.put(EImage.TIGER_ICON_32x32, new ImageIcon(IMAGE_PATH + "tigerIcon_32x32.png"));
		
		storage.put(EImage.TIGER_ICON_16x16, new ImageIcon(IMAGE_PATH + "tigerIcon_16x16.png"));
		
		storage.put(EImage.BULLET_WRENCH, new ImageIcon(IMAGE_PATH + "bullet_wrench.png"));
		
		storage.put(EImage.EXIT, new ImageIcon(IMAGE_PATH + "exit.png"));
	}
	

	@Override
	public ImageIcon getIcon(EImage key) throws SimResourceException {
		return getData(key);
	}
	

	@Override
	public Image getImage(EImage key) throws SimResourceException {
		return getData(key).getImage();
	}
	

	@Override
	public com.jme.image.Image getImageJME(EImage key) throws SimResourceException {
		
		final Image img = getData(key).getImage();
		return TextureManager.loadImage(img, false);
	}
}
