package edu.dhbw.mannheim.tigers.sim.util.network;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.NetworkInterface;
import java.util.List;

import edu.dhbw.mannheim.tigers.sim.util.collection.Pair;


/**
 * Combines a {@link MulticastUDPTransmitter} with an {@link MulticastUDPReceiver}
 * 
 * @author Gero
 */
public class MulticastUDPTransceiver implements ITransmitter<byte[]>, IReceiver {
	
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private final MulticastUDPTransmitter	trans;
	private final MulticastUDPReceiver		recv;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see MulticastUDPTransceiver
	 * @param localPort
	 * @param targetAddr
	 * @param targetPort
	 * @param pairs
	 */
	public MulticastUDPTransceiver(int localPort, String targetAddr, int targetPort,
			List<Pair<InetSocketAddress, NetworkInterface>> pairs) {
		trans = new MulticastUDPTransmitter(localPort, targetAddr, targetPort);
		recv = new MulticastUDPReceiver(trans.getSocket(), pairs);
	}
	
	
	/**
	 * @see MulticastUDPTransceiver
	 * @param localPort
	 * @param targetAddr
	 * @param targetPort
	 * @param nif
	 */
	public MulticastUDPTransceiver(int localPort, String targetAddr, int targetPort,
			NetworkInterface nif) {
		trans = new MulticastUDPTransmitter(localPort, targetAddr, targetPort);
		recv = new MulticastUDPReceiver(trans.getSocket(), targetAddr, nif);
	}
	

	/**
	 * @see MulticastUDPTransceiver
	 * @param localPort
	 * @param targetAddr
	 * @param targetPort
	 */
	public MulticastUDPTransceiver(int localPort, String targetAddr, int targetPort) {
		trans = new MulticastUDPTransmitter(localPort, targetAddr, targetPort);
		recv = new MulticastUDPReceiver(trans.getSocket());
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public boolean send(byte[] data) {
		return trans.send(data);
	}
	

	@Override
	public DatagramPacket receive(DatagramPacket store) throws IOException {
		return recv.receive(store);
	}
	

	@Override
	public void cleanup() throws IOException {
		trans.cleanup();
		recv.cleanup();
	}
	

	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public int getTargetPort() {
		return trans.getTargetPort();
	}
	

	@Override
	public int getLocalPort() {
		return recv.getLocalPort();
	}
	

	@Override
	public InetAddress getLocalAddress() {
		return recv.getLocalAddress();
	}
	

	@Override
	public boolean isReady() {
		return trans.isReady() && recv.isReady();
	}
}
