package edu.dhbw.mannheim.tigers.sim.util.network;

import java.io.IOException;
import java.net.BindException;
import java.net.DatagramPacket;
import java.net.InetAddress;


/**
 * Combines a {@link UnicastUDPTransmitter} with a {@link UnicastUDPReceiver}
 * 
 * @author Gero
 */
public class UnicastUDPTransceiver implements IReceiver, ITransmitter<byte[]> {
	// --------------------------------------------------------------------------
	// --- variables and constants ----------------------------------------------
	// --------------------------------------------------------------------------
	private final UnicastUDPTransmitter	trans;
	private final UnicastUDPReceiver		recv;
	
	
	// --------------------------------------------------------------------------
	// --- constructors ---------------------------------------------------------
	// --------------------------------------------------------------------------
	/**
	 * @see UnicastUDPTransceiver
	 * @param localPort
	 * @param localAddrStr
	 * @param targetAddrStr
	 * @param targetPort
	 * @throws BindException
	 */
	public UnicastUDPTransceiver(int localPort, String localAddrStr, String targetAddrStr, int targetPort)
			throws BindException {
		trans = new UnicastUDPTransmitter(localPort, localAddrStr, targetAddrStr, targetPort);
		recv = new UnicastUDPReceiver(trans.getSocket());
	}
	

	/**
	 * @see UnicastUDPTransceiver
	 * @param localPort
	 * @param targetAddrStr
	 * @param targetPort
	 * @throws BindException 
	 */
	public UnicastUDPTransceiver(int localPort, String targetAddrStr, int targetPort) throws BindException {
		this(localPort, null, targetAddrStr, targetPort);
	}
	

	// --------------------------------------------------------------------------
	// --- methods --------------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public boolean send(byte[] data) {
		return trans.send(data);
	}
	

	@Override
	public DatagramPacket receive(DatagramPacket store) throws IOException {
		return recv.receive(store);
	}
	

	@Override
	public void cleanup() throws IOException {
		trans.cleanup();
		recv.cleanup();
	}
	

	// --------------------------------------------------------------------------
	// --- getter/setter --------------------------------------------------------
	// --------------------------------------------------------------------------
	@Override
	public int getLocalPort() {
		return trans.getLocalPort();
	}
	

	@Override
	public int getTargetPort() {
		return trans.getTargetPort();
	}
	

	@Override
	public InetAddress getLocalAddress() {
		return trans.getLocalAddress();
	}
	

	@Override
	public boolean isReady() {
		return trans.isReady();
	}
}
